@@ -170,8 +170,6 @@ def obj_get_attr(obj: Atspi.Object, name:str) -> Either[str]:
         return obj.get_name() or ""
     elif name == 'text':
         return obj.get_text(0, -1)
-    # TODO: Fix this getattr(obj, name) may not work when adding objects to the tree
-    # TODO: Use the get_property method of GObject
     elif hasattr(obj, name):
         return getattr(obj, name)
     elif hasattr(obj, f"get_{name}"):
@@ -600,9 +598,9 @@ def perform_on_each(roots: Iterable[Atspi.Object], **kwargs: MatchArgs) -> UIMul
         If no object matches the patterns in `kwargs`
 
     """
-    on_objs = []
-    for root in roots:
-        on_objs.extend(find_all_objs(root, **kwargs))
+
+    on_objs = [ fail_on_error(find_obj(root, **kwargs))
+                for root in roots ]
 
     def do(action_name: str, **kwargs) -> None:
         for on_obj in on_objs:
